!
! Daily update of leaf area index etc.
!
!! $Header: /home/ssipsl/CVSREP/ORCHIDEE/src_sechiba/slowproc.f90,v 1.13.2.3 2004/01/13 10:59:44 ssipsl Exp $
!
!
MODULE slowproc

  ! modules used:

  USE reqdprec
  USE constantes 
  USE constantes_veg
  USE constantes_co2
  USE ioipsl
  USE sechiba_io
  USE stomate

  IMPLICIT NONE

  PRIVATE
  PUBLIC slowproc_main,slowproc_clear

  !
  ! variables used inside slowproc module : declaration and initialisation
  !

  LOGICAL, SAVE                                   :: l_first_slowproc = .TRUE.!! Initialisation has to be done one time
  LOGICAL, SAVE                                   :: read_lai =.FALSE.        !! By default, we read the LAI map for a global run
  REAL(stnd), SAVE                                :: day_counter              !! Counter to verify day change
  REAL(stnd), SAVE                                :: dt_slow                  !! time step of slow processes and STOMATE
  REAL(stnd), SAVE                                :: clayfraction_default = 0.2

  !

  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:)    :: clayfraction 
  REAL(stnd), ALLOCATABLE, SAVE, DIMENSION (:,:,:):: laimap 

  !

CONTAINS

  SUBROUTINE slowproc_main (kjit, kjpij, kjpindex, dtradia, date0,&
       ldrestart_read, ldrestart_write, ldforcing_write, ldcarbon_write, &
       index, lalo, neighbours, resolution, soiltype, &
       t2m, t2m_min, temp_sol, stempdiag, &
       humrel, shumdiag, litterhumdiag, precip_rain, precip_snow, gpp, &
       deadleaf_cover, &
       assim_param, &
       lai, height, veget, frac_nobio, veget_max, totfrac_nobio, qsintmax, &
       rest_id, hist_id, rest_id_stom, hist_id_stom, &
       co2_flux)


    ! interface description
    ! input scalar 
    INTEGER(long), INTENT(in)                          :: kjit             !! Time step number
    INTEGER(long), INTENT(in)                          :: kjpij            !! Total size of the un-compressed grid
    INTEGER(long),INTENT (in)                          :: kjpindex         !! Domain size
    REAL(stnd),INTENT (in)                             :: dtradia          !! Time step
    REAL(stnd),INTENT (in)                             :: date0            !! Initial date
    LOGICAL, INTENT(in)                                :: ldrestart_read   !! Logical for _restart_ file to read
    LOGICAL, INTENT(in)                                :: ldrestart_write  !! Logical for _restart_ file to write
    LOGICAL, INTENT(in)                                :: ldforcing_write  !! Logical for _forcing_ file to write
    LOGICAL, INTENT(in)                                :: ldcarbon_write   !! Logical for _carbon_forcing_ file to write
    INTEGER(long),INTENT (in)                          :: rest_id,hist_id  !! _Restart_ file and _history_ file identifier
    INTEGER(long),INTENT (in)                          :: rest_id_stom     !! STOMATE's _Restart_ file file identifier
    INTEGER(long),INTENT (in)                          :: hist_id_stom     !! STOMATE's _history_ file file identifier
    ! input fields
    INTEGER(long),DIMENSION (kjpindex), INTENT (in)    :: index            !! Indeces of the points on the map
    REAL(stnd),DIMENSION (kjpindex,2), INTENT (in)     :: lalo             !! Geogr. coordinates (latitude,longitude) (degrees)
    INTEGER(long), DIMENSION (kjpindex,8), INTENT(in)  :: neighbours       !! neighoring grid points if land
    REAL(stnd), DIMENSION (kjpindex,2), INTENT(in)     :: resolution       !! size in x an y of the grid (m)
    REAL(stnd), DIMENSION (kjpindex,nvm), INTENT (in)  :: humrel           !! Relative humidity ("moisture stress")
    REAL(stnd), DIMENSION(kjpindex), INTENT(in)        :: t2m              !! 2 m air temperature (K)
    REAL(stnd), DIMENSION(kjpindex), INTENT(in)        :: t2m_min          !! min. 2 m air temp. during forcing time step (K)
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)       :: temp_sol         !! Surface temperature
    REAL(stnd),DIMENSION (kjpindex,nbdl), INTENT (in)  :: stempdiag        !! Soil temperature
    REAL(stnd),DIMENSION (kjpindex,nbdl), INTENT (in)  :: shumdiag         !! Relative soil moisture
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)       :: litterhumdiag    !! Litter humidity
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)       :: precip_rain      !! Rain precipitation
    REAL(stnd),DIMENSION (kjpindex), INTENT (in)       :: precip_snow      !! Snow precipitation
    REAL(stnd), DIMENSION(kjpindex,nvm), INTENT(in)    :: gpp              !! GPP (gC/(m**2 of total ground)/time step)
    ! output scalar 
    ! output fields
    REAL(stnd), DIMENSION (kjpindex), INTENT(out)      :: co2_flux         !! CO2 flux in gC/m**2 of average ground/second
    ! modified scalar
    ! modified fields
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (inout):: lai              !! Surface foliere
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (inout):: height           !! height (m)
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (inout):: veget            !! Fraction of vegetation type 
    REAL(stnd),DIMENSION (kjpindex,nnobio), INTENT (inout):: frac_nobio    !! Fraction of ice, lakes, cities etc. in the mesh
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (inout):: veget_max        !! Fraction of vegetation type 
    REAL(stnd),DIMENSION (kjpindex), INTENT (inout)    :: totfrac_nobio    !! Total fraction of ice+lakes+cities etc. in the mesh
    REAL(stnd), DIMENSION (kjpindex,nbsoil), INTENT(inout):: soiltype      !! fraction of soil type
    REAL(stnd),DIMENSION (kjpindex,nvm,npco2), INTENT (inout):: assim_param!! min+max+opt temps, vcmax, vjmax for photosynthesis
    REAL(stnd),DIMENSION (kjpindex), INTENT (inout)    :: deadleaf_cover   !! fraction of soil covered by dead leaves
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (inout):: qsintmax         !! Maximum water on vegetation for interception

    ! local declaration
    INTEGER(short)                                     :: ji, jv, ier
    INTEGER(short), SAVE                               :: lcanop           !! soil level used for LAI
    REAL(stnd)                                         :: tmp_day(1)
    INTEGER(long)                                      :: yy, mm, dd
    REAL(stnd)                                         :: ss      
    REAL(stnd)                                         :: in_julian
    CHARACTER(LEN=80)                                       :: var_name         !! To store variables names for I/O
    LOGICAL                                            :: impveg
    REAL(stnd)                                         :: frac_nobio1        !! temporary
    REAL(stnd)                                         :: stempdiag_bid      !! only needed for an initial LAI if there is no

    !
    ! do initialisation
    !
    IF (l_first_slowproc) THEN

       !
       ! 1.1 allocation, file definitions. Restart file read for Sechiba. Set flags.
       !

       IF (long_print) WRITE (numout,*) ' l_first_slowproc : call slowproc_init '
       CALL slowproc_init (kjit, ldrestart_read, dtradia, date0, kjpindex, index, lalo, neighbours, resolution,&
            & rest_id, read_lai, lai, veget, frac_nobio, totfrac_nobio, soiltype, veget_max, height, lcanop)


       !
       ! 1.2 check time step
       !

       IF ( dt_slow .LT. dtradia ) THEN
          WRITE(numout,*) 'slow_processes: time step smaller than forcing time step.'
          STOP 'slowproc_main'
       ENDIF

       IF ( control%stomate_watchout .OR. control%ok_stomate ) THEN
          !
          ! 1.3 call STOMATE for initialization
          !
          CALL stomate_main (kjit, kjpij, kjpindex, dtradia, dt_slow, qsintcst, &
               ldrestart_read, ldrestart_write, ldforcing_write, ldcarbon_write, &
               index, lalo, neighbours, resolution, totfrac_nobio, clayfraction, &
               t2m, t2m_min, temp_sol, stempdiag, &
               humrel, shumdiag, litterhumdiag, precip_rain, precip_snow, gpp, &
               deadleaf_cover, &
               assim_param, &
               lai, height, veget, veget_max, qsintmax, &
               hist_id, rest_id_stom, hist_id_stom, &
               co2_flux)
          !
       ENDIF
       !
       IF ( .NOT. control%ok_stomate ) THEN
          !
          ! 1.4 initialize some variables
          !     STOMATE diagnoses some variables for SECHIBA: height, deadleaf_cover, etc.
          !     IF SECHIBA is not coupled to STOMATE, then we must set these values otherwise.
          !
          CALL slowproc_derivvar (kjpindex, veget, lai, &
               qsintmax, deadleaf_cover, assim_param, height)
       ENDIF

       RETURN

    ENDIF

    !
    ! prepares restart file for the next simulation
    !
    IF (ldrestart_write) THEN

       IF (long_print) WRITE (numout,*) ' we have to complete restart file with SLOWPROC variables '

       tmp_day(1) = day_counter
       var_name= 'day_counter'
       CALL restput (rest_id, var_name, 1 , 1  , 1, kjit, tmp_day)

       var_name= 'veget'
       CALL restput (rest_id, var_name, kjpindex, nvm, 1, kjit, veget, 'scatter',  kjpindex, index)
       !
       var_name= 'veget_max'
       CALL restput (rest_id, var_name, kjpindex, nvm, 1, kjit, veget_max, 'scatter',  kjpindex, index)
       !
       var_name= 'lai'
       CALL restput (rest_id, var_name, kjpindex, nvm, 1, kjit, lai, 'scatter',  kjpindex, index)
       !
       var_name= 'frac_nobio'
       CALL restput (rest_id, var_name, kjpindex, nnobio, 1, kjit, frac_nobio, 'scatter',  kjpindex, index)
       !
       var_name= 'soiltype_frac'
       CALL restput (rest_id, var_name, kjpindex, nbsoil, 1, kjit, soiltype, 'scatter',  kjpindex, index)
       !
       var_name= 'clay_frac'
       CALL restput (rest_id, var_name, kjpindex, 1, 1, kjit, clayfraction, 'scatter',  kjpindex, index)
       !
       ! The height of the vegetation could in principle be recalculated at the beginning of the run.
       ! However, this is very tedious, as many special cases have to be taken into account. This variable
       ! is therefore saved in the restart file.
       var_name= 'height'
       CALL restput (rest_id, var_name, kjpindex, nvm, 1, kjit, height, 'scatter',  kjpindex, index)

       IF (read_lai) THEN     
          var_name= 'laimap'
          CALL restput (rest_id, var_name, kjpindex, nvm, 12, kjit, laimap)
       ENDIF

       ! call STOMATE to write RESTART files
       IF ( control%stomate_watchout .OR. control%ok_stomate ) THEN
          CALL stomate_main (kjit, kjpij, kjpindex, dtradia, dt_slow, qsintcst, &
               ldrestart_read, ldrestart_write, ldforcing_write, ldcarbon_write, &
               index, lalo, neighbours, resolution, totfrac_nobio, clayfraction, &
               t2m, t2m_min, temp_sol, stempdiag, &
               humrel, shumdiag, litterhumdiag, precip_rain, precip_snow, gpp, &
               deadleaf_cover, &
               assim_param, &
               lai, height, veget, veget_max, qsintmax, &
               hist_id, rest_id_stom, hist_id_stom, &
               co2_flux)
       ENDIF

       RETURN

    END IF

    IF ( control%stomate_watchout .OR. control%ok_stomate ) THEN

       !
       ! 1 call STOMATE, either because we want to keep track of long-term variables or
       !   because STOMATE is activated
       !
       CALL stomate_main (kjit, kjpij, kjpindex, dtradia, dt_slow, qsintcst, &
            ldrestart_read, ldrestart_write, ldforcing_write, ldcarbon_write, &
            index, lalo, neighbours, resolution, totfrac_nobio, clayfraction, &
            t2m, t2m_min, temp_sol, stempdiag, &
            humrel, shumdiag, litterhumdiag, precip_rain, precip_snow, gpp, &
            deadleaf_cover, &
            assim_param, &
            lai, height, veget, veget_max, qsintmax, &
            hist_id, rest_id_stom, hist_id_stom, &
            co2_flux)

    ENDIF

    IF ( .NOT. control%ok_stomate ) THEN

       !
       ! 2 STOMATE is not activated: we have to guess lai etc.
       !

       !
       ! 2.1 update day counter
       !
       day_counter = day_counter + dtradia

       !
       ! 2.2 test if we have work to do
       !

       IF ( NINT(day_counter) .GE. NINT(dt_slow) ) THEN
          !
          day_counter = zero

          ! 2.2.1 do daily processes if necessary
          !
          IF (long_print) WRITE (numout,*) 'slowproc_main : We update the daily variables'
          ! 2.2.2 updates lai
          in_julian = itau2date(kjit, date0, dtradia)
          CALL ju2ymds(in_julian, yy, mm, dd, ss)
          CALL slowproc_lai (kjpindex, lcanop,stempdiag, &
               lai, mm, dd, read_lai, laimap)
          !
          WRITE(*,*) 'MINMAX LAI : ', MINVAL(lai), MAXVAL(lai)
          !
          ! 2.2.3 updates veget
          CALL slowproc_veget (kjpindex, lai, frac_nobio, veget_max, veget)

          ! 2.2.4 updates qsintmax and other derived variables
          CALL slowproc_derivvar (kjpindex, veget, lai, &
               qsintmax, deadleaf_cover, assim_param, height)

       END IF

       !
       !
       ! 2.3 some output fields
       !

       co2_flux(:) = 0.0

    ENDIF

    IF (long_print) WRITE (numout,*) ' slowproc_main done '

  END SUBROUTINE slowproc_main
  !!
  !!
  !!
  SUBROUTINE slowproc_init (kjit, ldrestart_read, dtradia, date0, kjpindex, index, lalo, neighbours, resolution,&
       & rest_id, read_lai, lai, veget, frac_nobio, totfrac_nobio, soiltype, veget_max, height, lcanop)

    ! interface description
    ! input scalar
    INTEGER(long), INTENT (in)                              :: kjit             !! Time step number
    LOGICAL, INTENT (in)                                    :: ldrestart_read   !! Logical for _restart_ file to read
    REAL(stnd),INTENT (in)                                  :: dtradia          !! Time step
    REAL(stnd), INTENT (in)                                 :: date0          !! intial date
    INTEGER(long), INTENT (in)                              :: kjpindex         !! Domain size
    INTEGER(long), INTENT (in)                              :: rest_id          !! _Restart_ file identifier
    ! input fields
    INTEGER(long),DIMENSION (kjpindex), INTENT (in)         :: index            !! Indeces of the points on the map
    REAL(stnd),DIMENSION (kjpindex,2), INTENT (in)          :: lalo             !! Geogr. coordinates (latitude,longitude) (degrees)
    INTEGER(long), DIMENSION (kjpindex,8), INTENT(in)       :: neighbours       !! neighoring grid points if land
    REAL(stnd), DIMENSION (kjpindex,2), INTENT(in)          :: resolution       !! size in x an y of the grid (m)
    ! output scalar
    INTEGER(short), INTENT(out)                             :: lcanop           !! soil level used for LAI
    ! output fields
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (out)       :: lai              !! Surface foliere
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (out)       :: veget            !! Fraction of vegetation type
    REAL(stnd),DIMENSION (kjpindex,nnobio), INTENT (out)    :: frac_nobio       !! Fraction of ice,lakes,cities, ...
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)           :: totfrac_nobio    !! Total fraction of ice+lakes+cities+...
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (out)       :: veget_max        !! Fraction of vegetation type
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (out)       :: height           !! Height of vegetation
    REAL(stnd), DIMENSION (kjpindex,nbsoil), INTENT(out)    :: soiltype         !! fraction of soil type
    ! local declaration
    REAL(stnd)                                              :: tmp_day(1)
    REAL(stnd)                                              :: zcanop             !! soil depth taken for canopy
    INTEGER(short)                                          :: vtmp(1)
    REAL(stnd), DIMENSION(nbdl)                             :: zsoil              !! soil depths at diagnostic levels
    INTEGER(short)                                          :: l                  !! Index
    CHARACTER(LEN=80)                                       :: var_name           !! To store variables names for I/O
    INTEGER(short)                                          :: ji, jv, ier
    LOGICAL                                                 :: impveg
    LOGICAL, INTENT (out)                                   :: read_lai
    REAL(stnd)                                              :: frac_nobio1        !! temporary
    REAL(stnd)                                              :: stempdiag_bid      !! only needed for an initial LAI if there is no
    !! restart file
    REAL(stnd)                                              :: in_julian
    INTEGER(long)                                           :: yy, mm, dd
    REAL(stnd)                                              :: ss    


    !
    ! 1 allocation
    !

    ALLOCATE (clayfraction(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in clayfraction allocation. We stop. We need kjpindex words = ',kjpindex
       STOP 'slowproc_init'
    END IF

    !
    ! 2 read restart file
    !

    var_name= 'day_counter'
    CALL ioconf_setatt('UNITS', 'd')
    CALL ioconf_setatt('LONG_NAME','Fraction of computed day')
    CALL restget (rest_id, var_name, 1       , 1  , 1, kjit, .TRUE., tmp_day)
    day_counter = tmp_day(1)

    ! get restart value if none were found in the restart file
    !
    !Config Key  = SECHIBA_DAY
    !Config Desc = Time within the day simulated
    !Config Def  = 0.0
    !Config Help = This is the time spent simulating the current day. This variable is
    !Config        prognostic as it will trigger all the computations which are
    !Config        only done once a day.
    !
    CALL setvar (day_counter, val_exp, 'SECHIBA_DAY', 0.0_stnd)
    !
    !Config  Key  = LAI_MAP
    !Config  Desc = Read the LAI map
    !Config Def  = n
    !Config Help = It is possible to read a 12 month LAI map which will
    !Config        then be interpolated to daily values as needed.
    !
    read_lai = .FALSE.
    CALL getin('LAI_MAP',read_lai)
    !
    !
    var_name= 'veget'
    CALL ioconf_setatt('UNITS', '-')
    CALL ioconf_setatt('LONG_NAME','Vegetation fraction')
    CALL restget (rest_id, var_name, kjpindex, nvm, 1, kjit, .TRUE., veget, "gather", kjpindex, index)
    !
    var_name= 'veget_max'
    CALL ioconf_setatt('UNITS', '-')
    CALL ioconf_setatt('LONG_NAME','Maximum vegetation fraction')
    CALL restget (rest_id, var_name, kjpindex, nvm, 1, kjit, .TRUE., veget_max, "gather", kjpindex, index)
    !
    var_name= 'frac_nobio'
    CALL ioconf_setatt('UNITS', '-')
    CALL ioconf_setatt('LONG_NAME','Special soil type fraction')
    CALL restget (rest_id, var_name, kjpindex, nnobio, 1, kjit, .TRUE., frac_nobio, "gather", kjpindex, index)
    !
    var_name= 'soiltype_frac'
    CALL ioconf_setatt('UNITS', '-')
    CALL ioconf_setatt('LONG_NAME','Fraction of each soil type')
    CALL restget (rest_id, var_name, kjpindex, nbsoil, 1, kjit, .TRUE., soiltype, "gather", kjpindex, index)
    !
    var_name= 'clay_frac'
    CALL ioconf_setatt('UNITS', '-')
    CALL ioconf_setatt('LONG_NAME','Fraction of clay in each mesh')
    CALL restget (rest_id, var_name, kjpindex, 1, 1, kjit, .TRUE., clayfraction, "gather", kjpindex, index)
    !
    var_name= 'lai'
    CALL ioconf_setatt('UNITS', '-')
    CALL ioconf_setatt('LONG_NAME','Leaf area index')
    CALL restget (rest_id, var_name, kjpindex, nvm, 1, kjit, .TRUE., lai, "gather", kjpindex, index)
    ! The height of the vegetation could in principle be recalculated at the beginning of the run.
    ! However, this is very tedious, as many special cases have to be taken into account. This variable
    ! is therefore saved in the restart file.

    var_name= 'height'
    CALL ioconf_setatt('UNITS', 'm')
    CALL ioconf_setatt('LONG_NAME','Height of vegetation')
    CALL restget (rest_id, var_name, kjpindex, nvm, 1, kjit, .TRUE., height, "gather", kjpindex, index)
    !
    !
    !
    IF (read_lai)THEN
       !
       ALLOCATE (laimap(kjpindex,nvm,12),stat=ier)
       IF (ier.NE.0) THEN
          WRITE (numout,*) ' error in laimap allocation. We stop. We need kjpindex*nvm*12 words = ',kjpindex*nvm*12
          STOP 'slowproc_init'
       END IF
       laimap(:,:,:) = val_exp
       !
       var_name= 'laimap'
       CALL ioconf_setatt('UNITS', '-')
       CALL ioconf_setatt('LONG_NAME','Leaf area index read')
       CALL restget (rest_id, var_name, kjpindex, nvm, 12, kjit , .TRUE., laimap)
       !
    ENDIF
    !
    !Config Key  = SECHIBA_ZCANOP
    !Config Desc = Soil level (m) used for canopy development (if STOMATE disactivated)
    !Config Def  = 0.5
    !Config Help = The temperature at this soil depth is used to determine the LAI when
    !Config        STOMATE is not activated.
    !
    zcanop = 0.5_stnd
    CALL setvar (zcanop, val_exp, 'SECHIBA_ZCANOP', 0.5_stnd)

    ! depth at center of the levels
    zsoil(1) = diaglev(1) / 2.
    DO l = 2, nbdl
       zsoil(l) = ( diaglev(l) - diaglev(l-1) ) / 2.
    ENDDO

    ! index of this level
    vtmp = MINLOC ( ABS ( zcanop - zsoil(:) ) )
    lcanop = vtmp(1)

    !
    !  Interception reservoir coefficient
    !
    !Config  Key  = 'SECHIBA_QSINT' 
    !Config  Desc = Interception reservoir coefficient
    !Config  Def  = 0.1
    !Config  Help = Transforms leaf area index into size of interception reservoir
    !Config         for slowproc_derivvar or stomate

    CALL getin('SECHIBA_QSINT', qsintcst)

    !
    ! Time step of STOMATE and LAI update
    !
    !Config  Key  = DT_SLOW
    !Config  Desc = Time step of STOMATE and other slow processes
    !Config  Def  = 86400.
    !Config  Help = Time step (s) of regular update of vegetation
    !Config         cover, LAI etc. This is also the time step
    !Config         of STOMATE.

    dt_slow = un_jour
    CALL getin('DT_SLOW', dt_slow)
    !
    !
    ! get restart value if none were found in the restart file
    !
    !
    !Config Key  = IMPOSE_VEG
    !Config Desc = Should the vegetation be prescribed
    !Config Def  = n
    !Config Help = This flag allows the user to impose a vegetation distribution
    !Config        and its characterisitcs. It is espacially interesting for 0D
    !Config        simulations. On the globe it does not make too much sense as
    !Config        it imposes the same vegetation everywhere
    !
    impveg = .FALSE.
    CALL getin ('IMPOSE_VEG', impveg)
    !
    IF ( impveg ) THEN
       !
       !  We are on a point and thus we can read the information from the run.def
       !
       !Config Key  = SECHIBA_VEG
       !Config Desc = Vegetation distribution within the mesh (0-dim mode)
       !Config If   = IMPOSE_VEG
       !Config Def  = 0.2, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.8, 0.0, 0.0, 0.0
       !Config Help = The fraction of vegetation is read from the restart file. If
       !Config        it is not found there we will use the values provided here.
       !
       CALL setvar (veget, val_exp, 'SECHIBA_VEG', veget_ori_fixed_test_1)

       !
       !Config Key  = SECHIBA_VEGMAX
       !Config Desc = Maximum vegetation distribution within the mesh (0-dim mode)
       !Config If   = IMPOSE_VEG
       !Config Def  = 0.2, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.8, 0.0, 0.0, 0.0
       !Config Help = The fraction of vegetation is read from the restart file. If
       !Config        it is not found there we will use the values provided here.
       !
       CALL setvar (veget_max, val_exp, 'SECHIBA_VEGMAX', veget_ori_fixed_test_1)

       !
       !Config Key  = SECHIBA_FRAC_NOBIO
       !Config Desc = Fraction of other surface types within the mesh (0-dim mode)
       !Config If   = IMPOSE_VEG
       !Config Def  = 0.0
       !Config Help = The fraction of ice, lakes, etc. is read from the restart file. If
       !Config        it is not found there we will use the values provided here.
       !Config        For the moment, there is only ice.
       !
       ! laisser ca tant qu'il n'y a que la glace. Pb avec setvar?
       frac_nobio1 = frac_nobio(1,1)
       CALL setvar (frac_nobio1, val_exp, 'SECHIBA_FRAC_NOBIO', frac_nobio_fixed_test_1)
       frac_nobio(:,:) = frac_nobio1
       ! CALL setvar (frac_nobio, val_exp, 'SECHIBA_FRAC_NOBIO', frac_nobio_fixed_test_1)

       !
       !Config Key  = SECHIBA_LAI
       !Config Desc = LAI for all vegetation types (0-dim mode)
       !Config Def  = 0., 8., 8., 4., 4.5, 4.5, 4., 4.5, 4., 2., 2., 2., 2.
       !Config If   = IMPOSE_VEG
       !Config Help = The maximum LAI used in the 0dim mode. The values should be found
       !Config        in the restart file. The new values of LAI will be computed anyway
       !Config        at the end of the current day. The need for this variable is caused
       !Config        by the fact that the model may stop during a day and thus we have not
       !Config        yet been through the routines which compute the new surface conditions.
       !
       CALL setvar (lai, val_exp, 'SECHIBA_LAI', llaimax)

       !Config Key  = SOIL_FRACTIONS
       !Config Desc = Fraction of the 3 soil types (0-dim mode)
       !Config Def  = 0.28, 0.52, 0.20
       !Config If   = IMPOSE_VEG
       !Config Help = Determines the fraction for the 3 soil types
       !Config        in the mesh in the following order : silt, sand and clay.
       !
       CALL setvar (soiltype, val_exp, 'SOIL_FRACTIONS', soiltype_default)

       !Config Key  = CLAY_FRACTION
       !Config Desc = Fraction of the clay fraction (0-dim mode)
       !Config Def  = 0.2
       !Config If   = IMPOSE_VEG
       !Config Help = Determines the fraction of clay in the grid box.
       !
       CALL setvar (clayfraction, val_exp, 'CLAY_FRACTION', clayfraction_default)

       !
       !Config Key  = SLOWPROC_HEIGHT
       !Config Desc = Height for all vegetation types (m)
       !Config Def  = 0., 30., 30., 20., 20., 20., 15., 15., 15., .5, .6, 1.0, 1.0
       !Config If   = IMPOSE_VEG
       !Config Help = The height used in the 0dim mode. The values should be found
       !Config        in the restart file. The new values of height will be computed anyway
       !Config        at the end of the current day. The need for this variable is caused
       !Config        by the fact that the model may stop during a day and thus we have not
       !Config        yet been through the routines which compute the new surface conditions.
       !
       CALL setvar (height, val_exp, 'SLOWPROC_HEIGHT', height_presc)

    ELSE
       !
       !  We are in the full 2-D case thus we need to do the interpolation if the potential vegetation
       !  is not available
       !
       IF ( ALL( veget_max(:,:) .EQ. val_exp ) .OR. ALL( frac_nobio(:,:) .EQ. val_exp ) ) THEN
          CALL slowproc_interpol(kjpindex, lalo, neighbours, resolution, veget_max, frac_nobio)
       ENDIF
       !
       IF (read_lai) THEN  
          !
          !  In case the LAI map was not found in the restart then we need to recompute it
          ! 
          IF ( ALL( laimap(:,:,:) .EQ. val_exp) ) THEN
             !
             CALL slowproc_interlai (kjpindex, lalo, resolution, laimap)
             !
          ENDIF
          !
       ENDIF
       !
       IF ( MINVAL(lai) .EQ. MAXVAL(lai) .AND. MAXVAL(lai) .EQ. val_exp) THEN
          !
          !  Get a first guess at the LAI
          !
          !Config Key  = SLOWPROC_LAI_TEMPDIAG
          !Config Desc = Temperature used for the initial guess of LAI
          !Config Def  = 280.
          !Config Help = If there is no LAI in the restart file, we may need
          !Config        a temperature that is used to guess the initial LAI.
          !
          CALL getin('SLOWPROC_LAI_TEMPDIAG',stempdiag_bid)
          stempdiag_bid = 280.
          !
          in_julian = itau2date(kjit, date0, dtradia)
          CALL ju2ymds(in_julian, yy, mm, dd, ss)
          !
          CALL slowproc_lai (kjpindex, lcanop, SPREAD(SPREAD(stempdiag_bid,1,kjpindex),2,nbdl), lai, mm,dd, read_lai, laimap)
          !
          !
          ! Make sure that we redo the computation when we will be back in slowproc_main
          day_counter = dt_slow + 1
       ENDIF

       IF ( MINVAL(veget) .EQ. MAXVAL(veget) .AND. MAXVAL(veget) .EQ. val_exp) THEN

          ! Impose height
          DO jv = 1, nvm
             height(:,jv) = height_presc(jv)
          ENDDO

          ! Have a first guess at the vegetation fraction
          CALL slowproc_veget (kjpindex, lai, frac_nobio, veget_max, veget)

          !  Make sure that we redo the computation when we will be back in slowproc_main
          day_counter = dt_slow + 1

       ENDIF

       IF ( MINVAL(soiltype) .EQ. MAXVAL(soiltype) .AND. MAXVAL(soiltype) .EQ. val_exp .OR.&
            & MINVAL(clayfraction) .EQ. MAXVAL(clayfraction) .AND. MAXVAL(clayfraction) .EQ. val_exp) THEN

          CALL slowproc_soilt(kjpindex, lalo, neighbours, resolution, soiltype, clayfraction)

       ENDIF

    ENDIF

    !
    ! calculate total fraction of the mesh that is covered by particular surface types: ice, lakes, ...
    !
    totfrac_nobio(:) = 0.
    DO jv = 1, nnobio
       totfrac_nobio(:) = totfrac_nobio(:) + frac_nobio(:,jv)
    ENDDO

    l_first_slowproc = .FALSE.

  END SUBROUTINE slowproc_init
  !!
  !! Clear Memory
  !!
  SUBROUTINE slowproc_clear 
    
 
    l_first_slowproc = .TRUE.
    IF (ALLOCATED (clayfraction)) DEALLOCATE (clayfraction)
    IF (ALLOCATED (laimap)) DEALLOCATE (laimap)
    !
    CALL stomate_clear 
    !
  END SUBROUTINE slowproc_clear
  !!
  !! Derive some variables
  !!
  SUBROUTINE slowproc_derivvar (kjpindex, veget, lai, &
                                qsintmax, deadleaf_cover, assim_param, height)

    ! interface description
    ! input scalar
    INTEGER(long), INTENT (in)                                 :: kjpindex      !! Domain size
    ! input fields
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)           :: veget         !! Fraction of vegetation type
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (in)           :: lai           !! Surface foliere
    ! output scalar
    ! output fields
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (out)          :: qsintmax      !! Maximum water on vegetation for interception
    REAL(stnd),DIMENSION (kjpindex), INTENT (out)              :: deadleaf_cover!! fraction of soil covered by dead leaves
    REAL(stnd), DIMENSION (kjpindex,nvm,npco2), INTENT (out)   :: assim_param   !! min+max+opt temps & vmax for photosynthesis
    REAL(stnd),DIMENSION (kjpindex,nvm), INTENT (out)        :: height          !! height (m)
    !
    ! local declaration
    !
    INTEGER(long)       :: ji, jv

    !
    ! 1 Assimilation parameters
    !
    DO jv = 1, nvm
      assim_param(:,jv,itmin) = co2_tmin_fix(jv) + tp_00
      assim_param(:,jv,itopt) = co2_topt_fix(jv) + tp_00
      assim_param(:,jv,itmax) = co2_tmax_fix(jv) + tp_00
      assim_param(:,jv,ivcmax) = vcmax_fix(jv)
      assim_param(:,jv,ivjmax) = vjmax_fix(jv)
    ENDDO

    !
    ! 2 fraction of soil covered by dead leaves
    !
    deadleaf_cover(:) = zero

    !
    ! 3 height
    !
    DO jv = 1, nvm
       height(:,jv) = height_presc(jv)
    ENDDO
    !
    ! 4 qsintmax
    !
    qsintmax(:,:) = qsintcst * veget(:,:) * lai(:,:)
    
  END SUBROUTINE slowproc_derivvar
  !!
  !!
  !!
  SUBROUTINE slowproc_mean (npts, n_dim2, dt_tot, dt, ldmean, field_in, field_mean)

    ! Accumulates field_in over a period of dt_tot.
    ! Has to be called at every time step (dt). 
    ! Mean value is calculated if ldmean=.TRUE.
    ! field_mean must be initialized outside of this routine!

    !
    ! 0 declarations
    !

    ! 0.1 input

    ! Domain size
    INTEGER, INTENT(in)                                 :: npts
    ! 2nd dimension (1 or npft)
    INTEGER, INTENT(in)                                 :: n_dim2
    ! Time step of STOMATE (days)
    REAL(stnd), INTENT(in)                              :: dt_tot
    ! Time step in days
    REAL(stnd), INTENT(in)                              :: dt
    ! Calculate mean ?
    LOGICAL, INTENT(in)                                 :: ldmean
    ! Daily field
    REAL(stnd), DIMENSION(npts,n_dim2), INTENT(in)      :: field_in

    ! 0.2 modified field

    ! Annual field
    REAL(stnd), DIMENSION(npts,n_dim2), INTENT(inout)   :: field_mean

    ! =========================================================================

    !
    ! 1 accumulation
    !

    field_mean(:,:) = field_mean(:,:) + field_in(:,:) * dt

    !
    ! 2 mean fields
    !

    IF (ldmean) THEN
      field_mean(:,:) = field_mean(:,:) / dt_tot
    ENDIF

  END SUBROUTINE slowproc_mean
  !!
  !!
  !!
  SUBROUTINE slowproc_long (npts, n_dim2, dt, tau, field_inst, field_long)

    ! Calculates a temporally smoothed field (field_long) from instantaneous 
    !   input fields.
    ! Time constant tau determines the strength of the smoothing.
    ! For tau -> infty, field_long becomes the true mean value of field_inst (but
    !   the spinup becomes infinietly long, too).
    ! field_long must be initialized outside of this routine!

    !
    ! 0 declarations
    !

    ! 0.1 input

    ! Domain size
    INTEGER, INTENT(in)                                 :: npts
    ! 2nd dimension (1 or npft)
    INTEGER, INTENT(in)                                 :: n_dim2
    ! Time step
    REAL(stnd), INTENT(in)                              :: dt
    ! Integration time constant (has to have same unit as dt!)
    REAL(stnd), INTENT(in)                              :: tau
    ! Instantaneous field
    REAL(stnd), DIMENSION(npts,n_dim2), INTENT(in)      :: field_inst

    ! 0.2 modified field

    ! Long-term field
    REAL(stnd), DIMENSION(npts,n_dim2), INTENT(inout)   :: field_long

    ! =========================================================================

    !
    ! 1 test coherence
    !

    IF ( ( tau .LT. dt ) .OR. ( dt .LE. 0. ) .OR. ( tau .LE. 0. ) ) THEN
      WRITE(numout,*) 'slowproc_long: Problem with time steps'
      WRITE(numout,*) 'dt=',dt
      WRITE(numout,*) 'tau=',tau
    ENDIF

    !
    ! 2 integration
    !

    field_long(:,:) = ( field_inst(:,:)*dt + field_long(:,:)*(tau-dt) ) / tau

  END SUBROUTINE slowproc_long
  !!
  !!
  !!
  SUBROUTINE slowproc_veget (kjpindex, lai, frac_nobio, veget_max, veget)
    !
    ! 0. Declarations
    !
    
    ! 0.1 Input
    INTEGER(long), INTENT(in)                             :: kjpindex
    REAL(stnd), DIMENSION(kjpindex,nvm), INTENT(in)       :: lai

    ! 0.2 Modified
    REAL(stnd), DIMENSION(kjpindex,nnobio), INTENT(inout) :: frac_nobio
    REAL(stnd), DIMENSION(kjpindex,nvm), INTENT(inout)    :: veget_max

    ! 0.3 Output
    REAL(stnd), DIMENSION(kjpindex,nvm), INTENT(out)      :: veget

    ! 0.4 Local
    REAL(stnd), DIMENSION(kjpindex)                       :: fracsum
    INTEGER(short)                                        :: nbad
    INTEGER(long)                                         :: ji, jv
    REAL(stnd)                                            :: trans_veg

    !
    ! 1. Sum up veget_max and frac_nobio and test if sum is equal to 1
    !

    !
    ! 1.1 Sum up
    !
    fracsum(:) = 0.
    DO jv = 1, nnobio
      DO ji = 1, kjpindex
        fracsum(ji) = fracsum(ji) + frac_nobio(ji,jv)
      ENDDO
    ENDDO
    DO jv = 1, nvm
      DO ji = 1, kjpindex
        fracsum(ji) = fracsum(ji) + veget_max(ji,jv)
      ENDDO
    ENDDO
    !
    ! 1.2 stop if there is a severe problem, that is an error above 0.01%
    !     The rest will be scaled
    !
    nbad = COUNT( ABS(fracsum(:)-un) .GT. 0.0001 )
    IF ( nbad .GT. 0 ) THEN
      WRITE(*,*) 'Problem with total surface areas.'
      DO ji = 1, kjpindex
        IF ( ABS(fracsum(ji)-un) .GT. 0.0001 ) THEN
          WRITE(*,*) 'Point :', ji
          WRITE(*,*) '  frac_nobio :', frac_nobio(ji,:)
          WRITE(*,*) '  Veget_max :', veget_max(ji,:)
          WRITE(*,*) '  Fracsum :', fracsum(ji), EPSILON(un)
          WRITE(*,*) '  The error is :', un - ( SUM(frac_nobio(ji,:)) + SUM(veget_max(ji,:)) )
          STOP 'slowproc_veget'
        ENDIF
      ENDDO
    ENDIF
    !
    ! 1.3 correction at places where the problem is surely precision-related
    !
    nbad = COUNT( ABS(fracsum(:)-un) .GT. EPSILON(un) )
    !
    IF ( nbad .GT. 0 ) THEN
      !
      IF ( long_print ) THEN
        WRITE(numout,*) 'WARNING! scaling frac_nobio and veget_max at', nbad, ' points'
      ENDIF
      !
      DO jv = 1, nnobio
        DO ji = 1, kjpindex
          IF ( ABS(fracsum(ji)-un) .GT. EPSILON(un) ) THEN
            frac_nobio(ji,jv) = frac_nobio(ji,jv)/fracsum(ji)
          ENDIF
        ENDDO
      ENDDO
      !
      DO jv = 1, nvm
        DO ji = 1, kjpindex
          IF ( ABS(fracsum(ji)-un) .GT. EPSILON(un) ) THEN
            veget_max(ji,jv) = veget_max(ji,jv)/fracsum(ji)
          ENDIF
        ENDDO
      ENDDO
      !
    ENDIF

    !
    ! 2. Set veget equal to veget_max
    !
    DO jv = 1, nvm
      DO ji = 1, kjpindex
        veget(ji,jv) = veget_max(ji,jv)
      ENDDO
    ENDDO

    !
    ! 3. if lai of a vegetation type (jv > 1) is small, increase soil part
    !

    DO jv = 1,nvm
      DO ji = 1, kjpindex

        IF ((jv .GT. 1) .AND. (lai(ji,jv) .LT. 0.5)) THEN
          trans_veg = 2.*(0.5 - lai(ji,jv)) * veget(ji,jv)
          ! We limit the smallest fraction to 0.5%
          IF (  veget(ji,jv) - trans_veg .GT. min_vegfrac ) THEN
            veget(ji,1) = veget(ji,1) + trans_veg
            veget(ji,jv) = veget(ji,jv) - trans_veg
          ELSE
            veget(ji,1) = veget(ji,1) + veget(ji,jv)
            veget(ji,jv) = zero
          ENDIF
        ENDIF

      ENDDO
    ENDDO

    !
    ! 4. Sum up surface fractions and test if the sum is equal to 1
    !

    !
    ! 4.1 Sum up
    !
    fracsum(:) = 0.
    DO jv = 1, nnobio
      DO ji = 1, kjpindex
        fracsum(ji) = fracsum(ji) + frac_nobio(ji,jv)
      ENDDO
    ENDDO
    DO jv = 1, nvm
      DO ji = 1, kjpindex
        fracsum(ji) = fracsum(ji) + veget_max(ji,jv)
      ENDDO
    ENDDO

    !
    ! 4.2 stop if there is a severe problem
    !
    nbad = COUNT( ABS(fracsum(:)-un) .GT. (REAL(nvm+nnobio,stnd)*EPSILON(un)) )
    !
    IF ( nbad .GT. 0 ) THEN
      WRITE(*,*) 'Problem with veget or frac_nobio.'
      DO ji = 1, kjpindex
        IF ( ABS(fracsum(ji)-un) .GT. (10.*EPSILON(un)) ) THEN
          WRITE(*,*) 'Point :', ji
          WRITE(*,*) '  frac_nobio :', frac_nobio(ji,:)
          WRITE(*,*) '  Veget :', veget(ji,:)
          WRITE(*,*) '  The error is :', un - (SUM(frac_nobio(ji,:)) + SUM(veget(ji,:)))
          STOP 'slowproc_veget'
        ENDIF
      ENDDO
    ENDIF

    ! 
    ! 4.3 correction at places where the problem is surely precision-related
    !
    nbad = COUNT( ABS(fracsum(:)-un) .GT. EPSILON(un) )
    !
    IF ( nbad .GT. 0 ) THEN
      !
      IF ( long_print ) THEN
        WRITE(numout,*) 'slowproc_veget: scaling veget at', nbad, ' points'
      ENDIF
      !
      DO jv = 1, nvm
        DO ji = 1, kjpindex
          IF ( ABS(fracsum(ji)-un) .GT. EPSILON(un) ) THEN
            veget(ji,jv) = veget(ji,jv) / fracsum(ji)
          ENDIF
        ENDDO
      ENDDO
      !
      DO jv = 1, nnobio
        DO ji = 1, kjpindex
          IF ( ABS(fracsum(ji)-un) .GT. EPSILON(un) ) THEN
            frac_nobio(ji,jv) = frac_nobio(ji,jv) / fracsum(ji)
          ENDIF
        ENDDO
      ENDDO
      !
    ENDIF

  END SUBROUTINE slowproc_veget
  !!
  !!
  !!
  SUBROUTINE slowproc_lai (kjpindex,lcanop,stempdiag, lai, mm, dd, read_lai, laimap)
    !
    ! 0. Declarations
    !

    ! 0.1 Input
    INTEGER(long), INTENT(in)                         :: kjpindex   !! Domain size
    INTEGER(short), INTENT(in)                        :: lcanop     !! soil level used for LAI
    INTEGER(long), INTENT(in)                         :: mm, dd  
    REAL(stnd),DIMENSION (kjpindex,nbdl), INTENT (in) :: stempdiag  !! Soil temperature
    REAL(stnd), DIMENSION(kjpindex,nvm,12), INTENT(in):: laimap      !! LAI lue
    LOGICAL, INTENT(in)                               :: read_lai
    ! 0.2 Output
    REAL(stnd), DIMENSION(kjpindex,nvm), INTENT(out)  :: lai      !! LAI

    ! 0.3 Local
    INTEGER(long)                                     :: ji,jv

    DO jv = 1,nvm

       SELECT CASE (type_of_lai(jv))

       CASE ("mean ")
          !
          ! 1. do the interpolation between laimax and laimin
          !
          DO ji = 1,kjpindex
             IF ( .NOT. read_lai ) THEN
                lai(ji,jv) = undemi * (llaimax(jv) + llaimin(jv))
             ELSE
                lai(ji,jv) = MAXVAL(laimap(ji,jv,:))
             ENDIF
          END DO

       CASE ("inter")
          !
          ! 2. do the interpolation between laimax and laimin
          !
          DO ji = 1,kjpindex
             IF ( .NOT. read_lai ) THEN
                lai(ji,jv) = llaimin(jv) + tempfunc(stempdiag(ji,lcanop)) * (llaimax(jv) - llaimin(jv))
             ELSE
                IF (mm .EQ. 1 ) THEN
                   IF (dd .LE. 15) THEN
                      lai(ji,jv) = laimap(ji,jv,12)*(1-(dd+15)/30.) + laimap(ji,jv,1)*((dd+15)/30.)
                   ELSE
                      lai(ji,jv) = laimap(ji,jv,1)*(1-(dd-15)/30.) + laimap(ji,jv,2)*((dd-15)/30.)
                   ENDIF
                ELSE IF (mm .EQ. 12) THEN
                   IF (dd .LE. 15) THEN
                      lai(ji,jv) = laimap(ji,jv,11)*(1-(dd+15)/30.) + laimap(ji,jv,12)*((dd+15)/30.)
                   ELSE
                      lai(ji,jv) = laimap(ji,jv,12)*(1-(dd-15)/30.) + laimap(ji,jv,1)*((dd-15)/30.)
                   ENDIF
                ELSE 
                   IF (dd .LE. 15) THEN
                      lai(ji,jv) = laimap(ji,jv,mm-1)*(1-(dd+15)/30.) + laimap(ji,jv,mm)*((dd+15)/30.)
                   ELSE
                      lai(ji,jv) = laimap(ji,jv,mm)*(1-(dd-15)/30.) + laimap(ji,jv,mm+1)*((dd-15)/30.)
                   ENDIF
                ENDIF
             ENDIF
          END DO

       CASE default
          !
          ! 3. Problem
          !
          WRITE (numout,*) 'This kind of lai choice is not possible. '// &
               ' We stop with type_of_lai ',jv,' = ', type_of_lai(jv) 
          STOP 'slowproc_lai'

       END SELECT

    END DO

  END SUBROUTINE slowproc_lai
  !!
  !! Interpolate the LAI map to the grid of the model
  !!
  SUBROUTINE slowproc_interlai(nbpt, lalo,  resolution, laimap)
    !
    !
    !
    !  0.1 INPUT
    !
    INTEGER(long), INTENT(in)          :: nbpt                  ! Number of points for which the data needs to be interpolated
    REAL(stnd), INTENT(in)             :: lalo(nbpt,2)          ! Vector of latitude and longitudes (beware of the order !)
    REAL(stnd), INTENT(in)             :: resolution(nbpt,2)    ! The size in km of each grid-box in X and Y
    !
    !  0.2 OUTPUT
    !
    REAL(stnd), INTENT(out)    ::  laimap(nbpt,nvm,12)         ! lai read variable and re-dimensioned
    !
    !  0.3 LOCAL
    !
    REAL(stnd), PARAMETER                          :: R_Earth = 6378000., min_sechiba=1.E-8
    !
    !
    CHARACTER(LEN=80) :: filename
    INTEGER(long) :: iml, jml, ijml, i, j, ik, lml, tml, fid, ib, jb,ip, jp, vid, ai,iki,jkj
    REAL(stnd) :: lev(1), date, dt, coslat, pi
    INTEGER(long) :: itau(1)
    REAL(stnd), ALLOCATABLE, DIMENSION(:,:) ::  mask_lu
    REAL(stnd), ALLOCATABLE, DIMENSION(:) :: lat_lu, lon_lu, mask
    REAL(stnd), ALLOCATABLE, DIMENSION(:) :: lat_ful, lon_ful
    REAL(stnd), ALLOCATABLE, DIMENSION(:,:,:) :: laimaporig
    REAL(stnd), ALLOCATABLE, DIMENSION(:,:,:,:) :: laimap_lu
    REAL(stnd), ALLOCATABLE, DIMENSION(:) :: lon_up, lon_low, lat_up, lat_low
    INTEGER, DIMENSION(nbpt) :: n_origlai
    INTEGER, DIMENSION(nbpt) :: n_found
    REAL(stnd), DIMENSION(nbpt,nvm) :: frac_origlai

    CHARACTER(LEN=80) :: meter
    REAL(stnd) :: prog, sumf
    LOGICAL :: found
    INTEGER(short) :: idi,jdi, ilast, jlast, jj, ii, jv, inear, iprog
    REAL(stnd) :: domaine_lon_min, domaine_lon_max, domaine_lat_min, domaine_lat_max
    !
    pi = 4. * ATAN(1.)
    !
    !
    !Config Key  = LAI_FILE
    !Config Desc = Name of file from which the vegetation map is to be read
    !Config If   = !LAI_MAP
    !Config Def  = ../surfmap/lai2D.nc
    !Config Help = The name of the file to be opened to read the LAI
    !Config        map is to be given here. Usualy SECHIBA runs with a 5kmx5km
    !Config        map which is derived from a Nicolas VIOVY one. 
    !
    filename = 'lai2D.nc'
    CALL getin('LAI_FILE',filename)
    !
    CALL flininfo(filename, iml, jml, lml, tml, fid)
    !
    !
    ALLOCATE(lon_lu(iml))
    ALLOCATE(lat_lu(jml))
    ALLOCATE(laimap_lu(iml,jml,nvm,tml))
    ALLOCATE(mask_lu(iml,jml))
    !
    WRITE(numout,*) 'slowproc_interlai : Reading the LAI file'
    !
    CALL flinget(fid, 'longitude', iml, 0, 0, 0, 1, 1, lon_lu)
    CALL flinget(fid, 'latitude', jml, 0, 0, 0, 1, 1, lat_lu)
    CALL flinget(fid, 'LAI', iml, jml, nvm, tml, 1, 12, laimap_lu)
    CALL flinget(fid, 'mask', iml, jml, 0, 0, 0, 1, mask_lu)

    !
    WRITE(numout,*) 'slowproc_interlai : ', lon_lu(1), lon_lu(iml),lat_lu(1), lat_lu(jml)
    !
    !
    ijml=iml*jml
    ALLOCATE(lon_ful(ijml))
    ALLOCATE(lat_ful(ijml))
    ALLOCATE(laimaporig(ijml,nvm,tml))
    ALLOCATE(mask(ijml))

    DO i=1,iml
       DO j=1,jml
          iki=(j-1)*iml+i
          lon_ful(iki)=lon_lu(i)
          lat_ful(iki)=lat_lu(j)
          laimaporig(iki,:,:)=laimap_lu(i,j,:,:)
          mask(iki)=mask_lu(i,j)
       ENDDO
    ENDDO
    !
    WHERE  ( laimaporig(:,:,:) .LT. 0 )
       laimaporig(:,:,:) = 0.
    ENDWHERE
    !
    !
    ALLOCATE(lon_up(nbpt)) 
    ALLOCATE(lon_low(nbpt))
    ALLOCATE(lat_up(nbpt))
    ALLOCATE(lat_low(nbpt))
    !
    DO ib =1, nbpt
       !
       !  We find the 4 limits of the grid-box. As we transform the resolution of the model
       !  into longitudes and latitudes we do not have the problem of periodicity.
       !  coslat is a help variable here !
       !
       coslat = MAX(COS(lalo(ib,1) * pi/180. ), 0.001 )*pi/180. * R_Earth
       !
       lon_up(ib) = lalo(ib,2) + resolution(ib,1)/(2.0*coslat) 
       lon_low(ib) = lalo(ib,2) - resolution(ib,1)/(2.0*coslat) 
       !
       coslat = pi/180. * R_Earth
       !
       lat_up(ib) = lalo(ib,1) + resolution(ib,2)/(2.0*coslat) 
       lat_low(ib) = lalo(ib,1) - resolution(ib,2)/(2.0*coslat) 
       !
       !
       !
    ENDDO
    !
    !  Get the limits of the integration domaine so that we can speed up the calculations
    !
    domaine_lon_min = MINVAL(lon_low)
    domaine_lon_max = MAXVAL(lon_up)
    domaine_lat_min = MINVAL(lat_low)
    domaine_lat_max = MAXVAL(lat_up)
    !
    !
    ilast = 1
    n_origlai(:) = 0
    laimap(:,:,:) = 0.    
    !
    DO ip=1,ijml
       !
       IF ( ( lon_ful(ip) .GE. domaine_lon_min ) .AND. &
            ( lon_ful(ip) .LE. domaine_lon_max ) .AND. &
            ( lat_ful(ip) .GE. domaine_lat_min ) .AND. &
            ( lat_ful(ip) .LE. domaine_lat_max )        ) THEN
          !
          ! look for point on GCM grid which this point on fine grid belongs to.
          ! First look at the point on the model grid where we arrived just before. There is 
          ! a good chace that neighbouring points on the fine grid fall into the same model
          ! grid box.
          !
          IF ( ( lon_ful(ip) .GE. lon_low(ilast) ) .AND. &
               ( lon_ful(ip) .LT. lon_up(ilast) ) .AND. &
               ( lat_ful(ip) .GE. lat_low(ilast) ) .AND. &
               ( lat_ful(ip) .LT. lat_up(ilast) )         ) THEN
             !
             ! We were lucky
             !
             IF (mask(ip) .GT. 0) THEN
                n_origlai(ilast) =  n_origlai(ilast) + 1  
                DO i=1,nvm
                   DO j=1,12   
                      laimap(ilast,i,j) = laimap(ilast,i,j) + laimaporig(ip,i,j)
                   ENDDO
                ENDDO
             ENDIF
             !
          ELSE
             !
             ! Otherwise, look everywhere.
             ! Begin close to last grid point.
             !
             found = .FALSE. 
             idi = 1
             !
             DO WHILE ( (idi .LT. nbpt) .AND. ( .NOT. found ) )

                !
                ! forward and backward
                !
                DO ii = 1,2
                   !
                   IF ( ii .EQ. 1 ) THEN
                      ib = ilast - idi
                   ELSE
                      ib = ilast + idi
                   ENDIF
                   !
                   IF ( ( ib .GE. 1 ) .AND. ( ib .LE. nbpt ) ) THEN 
                      IF ( ( lon_ful(ip) .GE. lon_low(ib) ) .AND. &
                           ( lon_ful(ip) .LT. lon_up(ib) ) .AND. &
                           ( lat_ful(ip) .GE. lat_low(ib) ) .AND. &
                           ( lat_ful(ip) .LT. lat_up(ib) )         ) THEN
                         !
                         IF (mask(ip) .gt. 0) THEN
                            DO i=1,nvm
                               DO j=1,12                
                                  laimap(ib,i,j) = laimap(ib,i,j) + laimaporig(ip,i,j) 
                               ENDDO
                            ENDDO
                            n_origlai(ib) =  n_origlai(ib) + 1
                         ENDIF
                         ilast = ib
                         found = .TRUE.
                         !
                      ENDIF
                   ENDIF
                   !
                ENDDO
                !
                idi = idi + 1
                !
             ENDDO
             !
          ENDIF ! lucky/not lucky
          !
       ENDIF     ! in the domain
    ENDDO


    ! determine fraction of LAI points in each box of the coarse grid
    DO ip=1,nbpt
       DO jv =1,nvm
          IF ( n_origlai(ip) .GT. 0 ) THEN
             laimap(ip,jv,:) = laimap(ip,jv,:)/REAL(n_origlai(ip),stnd)
          ELSE
             laimap(ip,jv,:) = 0.
          ENDIF
       ENDDO
    ENDDO
    ! 
    DO i=1,12
    ENDDO
    WRITE(numout,*) 'slowproc_interlai : Interpolation Done'
    !
    !  
    !
    DEALLOCATE(lon_up)
    DEALLOCATE(lon_low)
    DEALLOCATE(lat_up)
    DEALLOCATE(lat_low)
    DEALLOCATE(lat_ful)
    DEALLOCATE(lon_ful)
    DEALLOCATE(lat_lu)
    DEALLOCATE(lon_lu)
    DEALLOCATE(laimap_lu)
    DEALLOCATE(laimaporig)
    DEALLOCATE(mask_lu)
    DEALLOCATE(mask)
    !
    RETURN
    !
  END SUBROUTINE slowproc_interlai

  !!
  !! Interpolate the IGBP vegetation map to the grid of the model
  !!
  SUBROUTINE slowproc_interpol(nbpt, lalo, neighbours, resolution, veget, frac_nobio )
  !
    !
    !
    !  0.1 INPUT
    !
    INTEGER(long), INTENT(in)          :: nbpt                  ! Number of points for which the data needs to be interpolated
    REAL(stnd), INTENT(in)             :: lalo(nbpt,2)          ! Vector of latitude and longitudes (beware of the order !)
    INTEGER(long), INTENT(in)          :: neighbours(nbpt,8)    ! Vector of neighbours for each grid point 
                                                                ! (1=N, 2=NE, 3=E, 4=SE, 5=S, 6=SW, 7=W, 8=NW)
    REAL(stnd), INTENT(in)             :: resolution(nbpt,2)    ! The size in km of each grid-box in X and Y
    !
    !  0.2 OUTPUT
    !
    REAL(stnd), INTENT(out)    ::  veget(nbpt,nvm)         ! Vegetation fractions
    REAL(stnd), INTENT(out)    ::  frac_nobio(nbpt,nnobio) ! Fraction of the mesh which is covered by ice, lakes, ...
    !
    !  0.3 LOCAL
    !
    REAL(stnd), PARAMETER                          :: R_Earth = 6378000., min_sechiba=1.E-8
    INTEGER(long), PARAMETER                       :: nolson = 94  ! Number of Olson classes
    !
    !
    CHARACTER(LEN=80) :: filename
    INTEGER(long) :: iml, jml, lml, tml, fid, ib, ip, jp, vid
    REAL(stnd) :: lev(1), date, dt, coslat, pi
    INTEGER(long) :: itau(1)
    REAL(stnd), ALLOCATABLE, DIMENSION(:) :: lat_ful, lon_ful, vegmap
    REAL(stnd), ALLOCATABLE, DIMENSION(:) :: lon_up, lon_low, lat_up, lat_low
    INTEGER, DIMENSION(nbpt,nolson) :: n_origveg
    INTEGER, DIMENSION(nbpt) :: n_found
    REAL(stnd), DIMENSION(nbpt,nolson) :: frac_origveg
    REAL(stnd) :: vegcorr(nolson,nvm)
    REAL(stnd) :: nobiocorr(nolson,nnobio)
    CHARACTER(LEN=80) :: meter
    REAL(stnd) :: prog, sumf
    LOGICAL :: found
    INTEGER :: idi, ilast, ii, jv, inear, iprog
    REAL(stnd) :: domaine_lon_min, domaine_lon_max, domaine_lat_min, domaine_lat_max
    !
    pi = 4. * ATAN(1.)
    !
    CALL get_vegcorr (nolson,vegcorr,nobiocorr)
    !
    !Config Key  = VEGETATION_FILE
    !Config Desc = Name of file from which the vegetation map is to be read
    !Config If   = !IMPOSE_VEG
    !Config Def  = ../surfmap/carteveg5km.nc
    !Config Help = The name of the file to be opened to read the vegetation
    !Config        map is to be given here. Usualy SECHIBA runs with a 5kmx5km
    !Config        map which is derived from the IGBP one. We assume that we have
    !Config        a classification in 87 types. This is Olson modified by Viovy.
    !
    filename = '../surfmap/carteveg5km.nc'
    CALL getin('VEGETATION_FILE',filename)
    !
    CALL flininfo(filename, iml, jml, lml, tml, fid)
    !
    !
    ALLOCATE(lat_ful(iml))
    ALLOCATE(lon_ful(iml))
    ALLOCATE(vegmap(iml))
    !
    WRITE(numout,*) 'Reading the vegetation file'
    !
    CALL flinget(fid, 'longitude', iml, jml, lml, tml, 1, 1, lon_ful)
    CALL flinget(fid, 'latitude', iml, jml, lml, tml, 1, 1, lat_ful)
    CALL flinget(fid, 'vegetation_map', iml, jml, lml, tml, 1, 1, vegmap)
    !
    IF (MAXVAL(vegmap) .LT. nolson) THEN
      WRITE(*,*) 'WARNING -- WARNING'
      WRITE(*,*) 'The vegetation map has to few vegetation types.'
      WRITE(*,*) 'If you are lucky it will work but please check'
    ELSE IF ( MAXVAL(vegmap) .GT. nolson) THEN
      WRITE(*,*) 'More vegetation types in file than the code can'
      WRITE(*,*) 'deal with.: ',  MAXVAL(vegmap),  nolson
      STOP 'slowproc_interpol'
    ENDIF
    !
    ALLOCATE(lon_up(nbpt)) 
    ALLOCATE(lon_low(nbpt))
    ALLOCATE(lat_up(nbpt))
    ALLOCATE(lat_low(nbpt))
    !
    DO ib =1, nbpt
       !
       !  We find the 4 limits of the grid-box. As we transform the resolution of the model
       !  into longitudes and latitudes we do not have the problem of periodicity.
       !  coslat is a help variable here !
       !
       coslat = MAX(COS(lalo(ib,1) * pi/180. ), 0.001 )*pi/180. * R_Earth
       !
       lon_up(ib) = lalo(ib,2) + resolution(ib,1)/(2.0*coslat) 
       lon_low(ib) = lalo(ib,2) - resolution(ib,1)/(2.0*coslat) 
       !
       coslat = pi/180. * R_Earth
       !
       lat_up(ib) = lalo(ib,1) + resolution(ib,2)/(2.0*coslat) 
       lat_low(ib) = lalo(ib,1) - resolution(ib,2)/(2.0*coslat) 
       !
       !
       veget(ib,:) = 0.0
       frac_nobio (ib,:) = 0.0
       !
    ENDDO
    !
    !  Get the limits of the integration domaine so that we can speed up the calculations
    !
    domaine_lon_min = MINVAL(lon_low)
    domaine_lon_max = MAXVAL(lon_up)
    domaine_lat_min = MINVAL(lat_low)
    domaine_lat_max = MAXVAL(lat_up)
    !
!!$    WRITE(*,*) 'DOMAINE lon :', domaine_lon_min, domaine_lon_max
!!$    WRITE(*,*) 'DOMAINE lat :', domaine_lat_min, domaine_lat_max
    !
    ! Ensure that the fine grid covers the whole domain
    WHERE ( lon_ful(:) .LT. domaine_lon_min )
      lon_ful(:) = lon_ful(:) + 360.
    ENDWHERE
    !
    WHERE ( lon_ful(:) .GT. domaine_lon_max )
      lon_ful(:) = lon_ful(:) - 360.
    ENDWHERE
    !
    WRITE(numout,*) 'Interpolating the vegetation map :'
    WRITE(numout,'(2a40)')'0%--------------------------------------', &
                   & '------------------------------------100%'
    !
    ilast = 1
    n_origveg(:,:) = 0
    !
    DO ip=1,iml
      !
      !   Give a progress meter
      !
      ! prog = ip/float(iml)*79.
      !  IF ( ABS(prog - NINT(prog)) .LT. 1/float(iml)*79. ) THEN
      !   meter(NINT(prog)+1:NINT(prog)+1) = 'x'
      !   WRITE(numout, advance="no", FMT='(a)') ACHAR(13)
      !   WRITE(numout, advance="no", FMT='(a80)') meter
      ! ENDIF
      iprog = NINT(float(ip)/float(iml)*79.) - NINT(float(ip-1)/float(iml)*79.)
      IF ( iprog .NE. 0 ) THEN
        WRITE(numout,'(a1,$)') 'x'
      ENDIF
      !
      !  Only start looking for its place in the samler grid if we are within the domaine
      !  That should speed up things !
      !
      IF ( ( lon_ful(ip) .GE. domaine_lon_min ) .AND. &
           ( lon_ful(ip) .LE. domaine_lon_max ) .AND. &
           ( lat_ful(ip) .GE. domaine_lat_min ) .AND. &
           ( lat_ful(ip) .LE. domaine_lat_max )        ) THEN
        !
        ! look for point on GCM grid which this point on fine grid belongs to.
        ! First look at the point on the model grid where we arrived just before. There is 
        ! a good chace that neighbouring points on the fine grid fall into the same model
        ! grid box.
        !
        !
        ! THERE IS A BUG HERE !!! IF THE GCM GRID SITS ON THE DATE LINE WE WILL HAVE FOR INSTANCE
        ! LON_LOW = -182 AND LON_UP = -178. THUS WE WILL ONLY PICK UP HALF THE POINTS NEEDED.
        !
        IF ( ( lon_ful(ip) .GT. lon_low(ilast) ) .AND. &
             ( lon_ful(ip) .LT. lon_up(ilast) ) .AND. &
             ( lat_ful(ip) .GT. lat_low(ilast) ) .AND. &
             ( lat_ful(ip) .LT. lat_up(ilast) )         ) THEN
          !
          ! We were lucky
          !
          n_origveg(ilast,NINT(vegmap(ip))) = n_origveg(ilast,NINT(vegmap(ip))) + 1
          !
        ELSE
          !
          ! Otherwise, look everywhere.
          ! Begin close to last grid point.
          !
          found = .FALSE. 
          idi = 1
          !
          DO WHILE ( (idi .LT. nbpt) .AND. ( .NOT. found ) )
            !
            ! forward and backward
            !
            DO ii = 1,2
              !
              IF ( ii .EQ. 1 ) THEN
                ib = ilast - idi
              ELSE
                ib = ilast + idi
              ENDIF
              !
              IF ( ( ib .GE. 1 ) .AND. ( ib .LE. nbpt ) ) THEN
                IF ( ( lon_ful(ip) .GT. lon_low(ib) ) .AND. &
                     ( lon_ful(ip) .LT. lon_up(ib) ) .AND. &
                     ( lat_ful(ip) .GT. lat_low(ib) ) .AND. &
                     ( lat_ful(ip) .LT. lat_up(ib) )         ) THEN
                  !
                  n_origveg(ib,NINT(vegmap(ip))) = n_origveg(ib,NINT(vegmap(ip))) + 1
                  ilast = ib
                  found = .TRUE.
                  !
                ENDIF
              ENDIF
              !
            ENDDO
            !
            idi = idi + 1
            !
          ENDDO
          !
        ENDIF ! lucky/not lucky
        !
      ENDIF     ! in the domain
    ENDDO

    !
    ! Now we know how many points of which Olson type from the fine grid fall
    ! into each box of the (coarse) model grid: n_origveg(nbpt,nolson)
    !

    !
    ! determine number of points of the fine grid which fall into each box of the 
    ! coarse grid
    !
    DO ib = 1, nbpt
      n_found(ib) = SUM( n_origveg(ib,:) )
    ENDDO

    !
    ! determine fraction of Olson vegetation type in each box of the coarse grid
    !
    DO vid = 1, nolson
      WHERE ( n_found(:) .GT. 0 ) 
        frac_origveg(:,vid) =  REAL(n_origveg(:,vid),stnd) /  REAL(n_found(:),stnd)
      ELSEWHERE
         frac_origveg(:,vid) = 0.
      ENDWHERE
    ENDDO

    !
    ! now finally calculate coarse vegetation map
    ! Find which model vegetation corresponds to each Olson type 
    !
    DO vid = 1, nolson
      !
      DO jv = 1, nvm
        veget(:,jv) = veget(:,jv) + frac_origveg(:,vid) * vegcorr(vid,jv)
      ENDDO
      !
      DO jv = 1, nnobio
        frac_nobio(:,jv) = frac_nobio(:,jv) + frac_origveg(:,vid) * nobiocorr(vid,jv)
      ENDDO
      !
    ENDDO
    !
    !
    WRITE(numout,*)
    WRITE(numout,*) 'Interpolation Done'
    !
    !   Clean up the point of the map
    !
    DO ib = 1, nbpt
       !
       !  Let us see if all points found something in the 5km map !
       !
       IF ( n_found(ib) .EQ. 0 ) THEN
          !
          ! Now we need to handle some exceptions
          !
          IF ( lalo(ib,1) .LT. -56.0) THEN
             ! Antartica
             frac_nobio(ib,:) = 0.0
             frac_nobio(ib,iice) = 1.0
             veget(ib,:) = 0.0
             !
          ELSE IF ( lalo(ib,1) .GT. 70.0) THEN
             ! Artica
             frac_nobio(ib,:) = 0.0
             frac_nobio(ib,iice) = 1.0
             veget(ib,:) = 0.0
             !
          ELSE IF ( lalo(ib,1) .GT. 55.0 .AND. lalo(ib,2) .GT. -65.0 .AND. lalo(ib,2) .LT. -20.0) THEN
             ! Greenland
             frac_nobio(ib,:) = 0.0
             frac_nobio(ib,iice) = 1.0
             veget(ib,:) = 0.0
             !
          ELSE
             !
             WRITE(numout,*) 'PROBLEM, no point in the 5km map found for this grid box'
             WRITE(numout,*) 'Longitude range : ', lon_low(ib), lon_up(ib)
             WRITE(numout,*) 'Latitude range : ', lat_low(ib), lat_up(ib)
             !
             WRITE(numout,*) 'Looking for nearest point on the 5 km map'
             CALL slowproc_nearest (iml, lon_ful, lat_ful, &
                                    lalo(ib,2), lalo(ib,1), inear)
             WRITE(numout,*) 'Coordinates of the nearest point:', &
                              lon_ful(inear),lat_ful(inear)
             !
             DO jv = 1, nvm
               veget(ib,jv) = vegcorr(NINT(vegmap(inear)),jv)
             ENDDO
             !
             DO jv = 1, nnobio
               frac_nobio(ib,jv) = nobiocorr(NINT(vegmap(inear)),jv)
             ENDDO
             !
          ENDIF
          !
       ENDIF
       !
       !
       !  Limit the smalest vegetation fraction to 0.5%
       !
       DO vid = 1, nvm
          IF ( veget(ib,vid) .LT. min_vegfrac ) THEN
             veget(ib,vid) = 0.0
          ENDIF
       ENDDO
       !
       sumf = SUM(frac_nobio(ib,:))+SUM(veget(ib,:))
       frac_nobio(ib,:) = frac_nobio(ib,:)/sumf
       veget(ib,:) = veget(ib,:)/sumf
       !
       !       
    ENDDO
    !
    DEALLOCATE(lon_up)
    DEALLOCATE(lon_low)
    DEALLOCATE(lat_up)
    DEALLOCATE(lat_low)
    DEALLOCATE(lat_ful)
    DEALLOCATE(lon_ful)
    DEALLOCATE(vegmap)
    !
    RETURN
    !
 END SUBROUTINE slowproc_interpol
  !!
  !! looks for nearest grid point on the fine map
  !!
  SUBROUTINE slowproc_nearest(iml, lon5, lat5, lonmod, latmod, inear)

    INTEGER(long), INTENT(in)                   :: iml
    REAL(stnd), DIMENSION(iml), INTENT(in)      :: lon5, lat5
    REAL(stnd), INTENT(in)                      :: lonmod, latmod

    INTEGER(long), INTENT(out)                  :: inear

    REAL(stnd)                                  :: pi
    REAL(stnd)                                  :: pa, p
    REAL(stnd)                                  :: coscolat, sincolat
    REAL(stnd)                                  :: cospa, sinpa
    REAL(stnd), DIMENSION(iml)                  :: cosang
    INTEGER(long)                               :: i
    INTEGER(long), DIMENSION(1)                 :: ineartab

    pi = 4.0 * ATAN(1.0)

    pa = pi/2.0 - latmod*pi/180.0 ! dist. entre pole n et point a
    cospa = COS(pa)
    sinpa = SIN(pa)

    DO i = 1, iml

      sincolat = SIN( pi/2.0 - lat5(i)*pi/180.0 ) 
      coscolat = COS( pi/2.0 - lat5(i)*pi/180.0 ) 

      p = (lonmod-lon5(i))*pi/180.0 ! angle entre a et b (leurs meridiens)

      ! dist(i) = ACOS( cospa*coscolat + sinpa*sincolat*COS(p))
      cosang(i) = cospa*coscolat + sinpa*sincolat*COS(p)

    ENDDO

    ineartab = MAXLOC( cosang(:) )
    inear = ineartab(1)

  END SUBROUTINE slowproc_nearest

  !!
  !! Interpolate the Zobler soil type map
  !!
  SUBROUTINE slowproc_soilt(nbpt, lalo, neighbours, resolution, soiltype, clayfraction)
    !
    !
    !   This subroutine should read the Zobler map and interpolate to the model grid. The method
    !   is to get fraction of the three main soiltypes for each grid box.
    !   The soil fraction are going to be put into the array soiltype in the following order :
    !   coarse, medium and fine.
    !
    !
    !  0.1 INPUT
    !
    INTEGER(long), INTENT(in)    :: nbpt                   ! Number of points for which the data needs to be interpolated
    REAL(stnd), INTENT(in)       :: lalo(nbpt,2)           ! Vector of latitude and longitudes (beware of the order !)
    INTEGER(long), INTENT(in)    :: neighbours(nbpt,8)     ! Vector of neighbours for each grid point 
    ! (1=N, 2=NE, 3=E, 4=SE, 5=S, 6=SW, 7=W, 8=NW)
    REAL(stnd), INTENT(in)       :: resolution(nbpt,2)     ! The size in km of each grid-box in X and Y
    REAL(stnd), INTENT(out)      :: soiltype(nbpt, nbsoil) ! Soil type map to be created from the Zobler map
    REAL(stnd), INTENT(out)      :: clayfraction(nbpt)     ! The fraction of clay as used by STOMATE
    !
    !
    !  0.3 LOCAL
    !
    INTEGER(long), PARAMETER :: nbvmax=200
    REAL(stnd), PARAMETER                          :: R_Earth = 6378000.
    !
    CHARACTER(LEN=80) :: filename
    INTEGER(long) :: iml, jml, lml, tml, fid, ib, ip, jp, fopt, ilf, lastjp, nbexp
    REAL(stnd) :: lev(1), date, dt, coslat, pi
    INTEGER(long) :: itau(1)
    REAL(stnd), ALLOCATABLE, DIMENSION(:,:) :: lat_rel, lon_rel, lat_ful, lon_ful, soiltext
    REAL(stnd), ALLOCATABLE, DIMENSION(:,:) :: loup_rel, lolow_rel, laup_rel, lalow_rel
    REAL(stnd) :: lon_up, lon_low, lat_up, lat_low
    INTEGER(long) :: solt(nbvmax)
    REAL(stnd) :: area(nbvmax), ax, ay, sgn
    REAL(stnd) :: lonrel, louprel, lolowrel
    !
    ! Number of texture classes in Zobler
    !
    INTEGER(long), PARAMETER :: classnb = 7
    REAL(stnd)               :: textfrac_table(classnb, nbsoil)
    !   
    !
    !
    pi = 4. * ATAN(1.)
    !
    CALL get_soilcorr (classnb, textfrac_table)
    !
    !  Needs to be a configurable variable
    !
    !
    !Config Key  = SOILTYPE_FILE
    !Config Desc = Name of file from which soil types are read
    !Config Def  = ../surfmap/soils_param.nc
    !Config If   = !IMPOSE_VEG
    !Config Help = The name of the file to be opened to read the soil types. 
    !Config        The data from this file is then interpolated to the grid of
    !Config        of the model. The aim is to get fractions for clay, sand and
    !Config        silt in each grid box. This information is used for soil hydrology
    !Config        and respiration.
    !
    filename = '../surfmap/soils_param.nc'
    CALL getin('SOILTYPE_FILE',filename)
    !
    CALL flininfo(filename,iml, jml, lml, tml, fid)
    !
    !
    ALLOCATE (lat_rel(iml,jml))
    ALLOCATE (lon_rel(iml,jml))
    ALLOCATE (laup_rel(iml,jml))
    ALLOCATE (loup_rel(iml,jml))
    ALLOCATE (lalow_rel(iml,jml))
    ALLOCATE (lolow_rel(iml,jml))
    ALLOCATE (lat_ful(iml+2,jml+2))
    ALLOCATE (lon_ful(iml+2,jml+2))
    ALLOCATE (soiltext(iml,jml))
    !
    CALL flinopen(filename, .FALSE., iml, jml, lml, lon_rel, lat_rel, lev, tml, itau, date, dt, fid)
    !
    CALL flinget(fid, 'soiltext', iml, jml, lml, tml, 1, 1, soiltext)
    !
    CALL flinclo(fid)
    !
    nbexp = 0
    !
    !    Duplicate the border assuming we have a global grid going from west to east
    !
    lon_ful(2:iml+1,2:jml+1) = lon_rel(1:iml,1:jml)
    lat_ful(2:iml+1,2:jml+1) = lat_rel(1:iml,1:jml)
    !
    IF ( lon_rel(iml,1) .LT. lon_ful(2,2)) THEN
       lon_ful(1,2:jml+1) = lon_rel(iml,1:jml)
       lat_ful(1,2:jml+1) = lat_rel(iml,1:jml)
    ELSE
       lon_ful(1,2:jml+1) = lon_rel(iml,1:jml)-360
       lat_ful(1,2:jml+1) = lat_rel(iml,1:jml)
    ENDIF

    IF ( lon_rel(1,1) .GT. lon_ful(iml+1,2)) THEN
       lon_ful(iml+2,2:jml+1) = lon_rel(1,1:jml)
       lat_ful(iml+2,2:jml+1) = lat_rel(1,1:jml)
    ELSE
       lon_ful(iml+2,2:jml+1) = lon_rel(1,1:jml)+360
       lat_ful(iml+2,2:jml+1) = lat_rel(1,1:jml)
    ENDIF
    !
    sgn = lat_rel(1,1)/ABS(lat_rel(1,1))
    lat_ful(2:iml+1,1) = sgn*180 - lat_rel(1:iml,1)
    sgn = lat_rel(1,jml)/ABS(lat_rel(1,jml))
    lat_ful(2:iml+1,jml+2) = sgn*180 - lat_rel(1:iml,jml)
    lat_ful(1,1) = lat_ful(iml+1,1)
    lat_ful(iml+2,1) = lat_ful(2,1)
    lat_ful(1,jml+2) = lat_ful(iml+1,jml+2)
    lat_ful(iml+2,jml+2) = lat_ful(2,jml+2)
    !
    ! Add the longitude lines to the top and bottom
    !
    lon_ful(:,1) = lon_ful(:,2) 
    lon_ful(:,jml+2) = lon_ful(:,jml+1) 
    !
    !  Get the upper and lower limits of each grid box
    !
    DO ip=1,iml
       DO jp=1,jml
          loup_rel(ip,jp) =MAX(0.5*(lon_ful(ip,jp+1)+lon_ful(ip+1,jp+1)), 0.5*(lon_ful(ip+1,jp+1)+lon_ful(ip+2,jp+1)))
          lolow_rel(ip,jp) =MIN(0.5*(lon_ful(ip,jp+1)+lon_ful(ip+1,jp+1)), 0.5*(lon_ful(ip+1,jp+1)+lon_ful(ip+2,jp+1)))
          laup_rel(ip,jp) =MAX(0.5*(lat_ful(ip+1,jp)+lat_ful(ip+1,jp+1)), 0.5*(lat_ful(ip+1,jp+1)+lat_ful(ip+1,jp+2)))
          lalow_rel(ip,jp) =MIN(0.5*(lat_ful(ip+1,jp)+lat_ful(ip+1,jp+1)), 0.5*(lat_ful(ip+1,jp+1)+lat_ful(ip+1,jp+2)))
       ENDDO
    ENDDO
    !
    !   Now we take each grid point and find out which values from the forcing we need to average
    !
    DO ib =1, nbpt
       !
       !  We find the 4 limits of the grid-box. As we transform the resolution of the model
       !  into longitudes and latitudes we do not have the problem of periodicity.
       ! coslat is a help variable here !
       !
       coslat = MAX(COS(lalo(ib,1) * pi/180. ), 0.001 )*pi/180. * R_Earth
       !
       lon_up = lalo(ib,2)+ resolution(ib,1)/(2.0*coslat) 
       lon_low =lalo(ib,2) - resolution(ib,1)/(2.0*coslat) 
       !
       coslat = pi/180. * R_Earth
       !
       lat_up =lalo(ib,1)+resolution(ib,2)/(2.0*coslat) 
       lat_low =lalo(ib,1)-resolution(ib,2)/(2.0*coslat) 
       !
       !
       !  Find the grid boxes from the data that go into the model's boxes
       !  We still work as if we had a regular grid ! Well it needs to be localy regular so
       !  so that the longitude at the latitude of the last found point is close to the one of the next point.
       !
       fopt = 0
       lastjp = 1
       DO ip=1,iml
          !
          !  Either the center of the data grid point is in the interval of the model grid or
          !  the East and West limits of the data grid point are on either sides of the border of
          !  the data grid.
          !
          !
          !  To do that correctly we have to check if the grid box sits on the date-line.
          !
          IF ( lon_low < -180.0 ) THEN
             lonrel = MOD( lon_rel(ip,lastjp) - 360.0, 360.0)
             lolowrel = MOD( lolow_rel(ip,lastjp) - 360.0, 360.0)
             louprel = MOD( loup_rel(ip,lastjp) - 360.0, 360.0)
             !
          ELSE IF ( lon_up > 180.0 ) THEN
             lonrel = MOD( 360. - lon_rel(ip,lastjp), 360.0)
             lolowrel = MOD( 360. - lolow_rel(ip,lastjp), 360.0)
             louprel = MOD( 360. - loup_rel(ip,lastjp), 360.0)
          ELSE
             lonrel = lon_rel(ip,lastjp)
             lolowrel = lolow_rel(ip,lastjp)
             louprel = loup_rel(ip,lastjp)
          ENDIF
          !
          !
          !
          IF ( lonrel > lon_low .AND. lonrel < lon_up .OR. &
               & lolowrel < lon_low .AND.  louprel > lon_low .OR. &
               & lolowrel < lon_up  .AND.  louprel > lon_up ) THEN
             !  
             DO jp = 1, jml
                !
                ! Now that we have the longitude let us find the latitude
                !
                IF ( lat_rel(ip,jp) > lat_low .AND. lat_rel(ip,jp) < lat_up .OR. &
                     & lalow_rel(ip,jp) < lat_low .AND. laup_rel(ip,jp) > lat_low .OR.&
                     & lalow_rel(ip,jp) < lat_up .AND. laup_rel(ip,jp) > lat_up) THEN
                   !
                   lastjp = jp
                   !
                   IF (soiltext(ip,jp) .GT. min_sechiba) THEN
                      !
                      fopt = fopt + 1
                      IF ( fopt .GT. nbvmax) THEN
                         WRITE(*,*) 'Please increase nbvmax in subroutine albedo_interpol', ib
                           STOP
                        ELSE
                           !
                           ! If we sit on the date line we need to do the same transformations as above.
                           ! 
                           IF ( lon_low < -180.0 ) THEN
                              lolowrel = MOD( lolow_rel(ip,jp) - 360.0, 360.0)
                              louprel = MOD( loup_rel(ip,jp) - 360.0, 360.0)
                              !
                           ELSE IF ( lon_up > 180.0 ) THEN
                              lolowrel = MOD( 360. - lolow_rel(ip,jp), 360.0)
                              louprel = MOD( 360. - loup_rel(ip,jp), 360.0)
                           ELSE
                              lolowrel = lolow_rel(ip,jp)
                              louprel = loup_rel(ip,jp)
                           ENDIF
                           !
                           ! Get the area of the fine grid in the model grid
                           !
                           coslat = MAX( COS( lat_rel(ip,jp) * pi/180. ), 0.001 )
                           ax = (MIN(lon_up,louprel)-MAX(lon_low, lolowrel))*pi/180. * R_Earth * coslat
                           ay = (MIN(lat_up, laup_rel(ip,jp))-MAX(lat_low,lalow_rel(ip,jp)))*pi/180. * R_Earth
                           area(fopt) = ax*ay
                           solt(fopt) = NINT(soiltext(ip,jp))
                        ENDIF
                     ENDIF
                     !
                  ENDIF
                  !
               ENDDO
               !
            ENDIF
            !
         ENDDO
         !
         !    Check that we found some points
         !
         soiltype(ib,:) = 0.0
         clayfraction(ib) = 0.0
         !
         IF ( fopt .EQ. 0) THEN
            nbexp = nbexp + 1
            soiltype(ib,:) = soiltype_default(:)
            clayfraction(ib) = clayfraction_default
         ELSE
            sgn = zero
            !
            !   Compute the average bare soil albedo parameters
            !
            DO ilf = 1,fopt
               !
               ! We have to take care of two exceptions here : type 6 = glacier and type 0 = ocean
               !
               IF ( (solt(ilf) .LE. classnb) .AND. (solt(ilf) .GT. 0) .AND.&
                    & (solt(ilf) .NE. 6)) THEN
                  SELECTCASE(solt(ilf))
                  CASE(1)
                     soiltype(ib,1) = soiltype(ib,1) + area(ilf)
                  CASE(2)
                     soiltype(ib,2) = soiltype(ib,2) + area(ilf)
                  CASE(3)
                     soiltype(ib,2) = soiltype(ib,2) + area(ilf)
                  CASE(4)
                     soiltype(ib,2) = soiltype(ib,2) + area(ilf)
                  CASE(5)
                     soiltype(ib,3) = soiltype(ib,3) + area(ilf)
                  CASE(7)
                     soiltype(ib,2) = soiltype(ib,2) + area(ilf)
                  CASE DEFAULT
                     WRITE(*,*) 'We should not be here, an impossible case appeared'
                     STOP 'slowproc_soilt'
                  END SELECT
                  clayfraction(ib) = clayfraction(ib) + textfrac_table(solt(ilf),3) * area(ilf)
                  sgn = sgn + area(ilf)
               ELSE
                  IF (solt(ilf) .GT. classnb) THEN
                     WRITE(*,*) 'The file contains a soil color class which is incompatible with this program'
                     STOP
                  ENDIF
               ENDIF
               !
            ENDDO
            !
            ! Normalize the surface
            !
            IF ( sgn .LT. min_sechiba) THEN
               nbexp = nbexp + 1
               soiltype(ib,:) = soiltype_default(:)
               clayfraction(ib) = clayfraction_default
            ELSE
               soiltype(ib,:) = soiltype(ib,:)/sgn
               clayfraction(ib) = clayfraction(ib)/sgn
            ENDIF
            !
         ENDIF
         !
      ENDDO
      !
      IF ( nbexp .GT. 0 ) THEN
         WRITE(*,*) 'slowproc_soilt : The interpolation of the bare soil albedo had ', nbexp
         WRITE(*,*) 'slowproc_soilt : points without data. This are either coastal points or'
         WRITE(*,*) 'slowproc_soilt : ice covered land.'
         WRITE(*,*) 'slowproc_soilt : The problem was solved by using the default soil types.'
      ENDIF
      !
      DEALLOCATE (lat_rel)
      DEALLOCATE (lon_rel)
      DEALLOCATE (laup_rel)
      DEALLOCATE (loup_rel)
      DEALLOCATE (lalow_rel)
      DEALLOCATE (lolow_rel)
      DEALLOCATE (lat_ful)
      DEALLOCATE (lon_ful)
      DEALLOCATE (soiltext)
      !
      !
      RETURN
      !
    END SUBROUTINE slowproc_soilt
    !


  END MODULE slowproc
